#!/bin/bash
#============================================================
#          FILE:  sourceConvertor.sh
#         USAGE:  ./sourceConvertor.sh --help, --version, --sourceroot
#   DESCRIPTION:  converts source code tree *.c *.cpp *h *.diff and *.lua to unix line endings
#       OPTIONS:  sourceroot is optional - if not supplied ./ is assumed to be source code tree root
#  REQUIREMENTS:  dos2unix is a dependency - install with sudo apt-get install dos2unix
#          BUGS:  ---
#         NOTES:  ---
#        AUTHOR:  Gijsbrecht De Waegeneer
#       COMPANY:  none
#       VERSION:  0.3
#       CREATED:  Mon 28 May 10:11:22 CEST 2018
#      REVISION:  2
#============================================================

# begin script
help="--help -display help, 
--version -display version, 
--sourceroot (optional - default: ./)"
version="0.1"
sourceroot="./"
program="/usr/bin/dos2unix"
sourcerootError="sourceroot $1 does not exist or is inaccesible"  
dependencyError="dos2unix not found - install with sudo apt-get install dos2unix"

function isDirProcessable(){
  #check if dir exists and if it is readable
  [[ -d "$1" && -r "$1" ]] && return #fastest way to combine multiple checks in one statement
}

function isDependencyOk(){
  [[  -f "$1" && -r "$1" && -x "$1" ]] && return
}

function isRoot(){
  [ "$(id -u)" == "0" ] && return
}

function processFiles(){
  SECONDS=0
  OLDIFS=$IFS
  IFS=$'\n'
  intC=0
  intCpp=0
  intH=0
  intDiff=0
  intLua=0
  intAnomaly=0
  red=$'\e[1;31m'
  grn=$'\e[1;32m'
  yel=$'\e[1;33m'
  blu=$'\e[1;34m'
  mag=$'\e[1;35m'
  cyn=$'\e[1;36m'
  end=$'\e[0m'
  regex="(.*\/)(.*\.)(.*)"
  echo "Ok go..."
  for line in `find $1 -name '*.c' -o -name '*.cpp' -o -name '*.h' -o -name '*.diff' -o -name '*.lua'`
  do  
    [[ $line =~ $regex ]]
    #echo "path: ${BASH_REMATCH[1]} file: ${BASH_REMATCH[2]} extension: ${BASH_REMATCH[3]}"
    case "${BASH_REMATCH[3]}" in
    "c")
      intC=`expr $intC + 1`
      ;;
    "cpp")
      intCpp=`expr $intCpp + 1`
      ;;
    "h")
      intH=`expr $intH + 1`
      ;;
    "diff")
      intDiff=`expr $intDiff + 1`
      ;;
    "lua")
      intLua=`expr $intLua + 1`
      ;;
    *)
      #default - can never happen
      intAnomaly=`expr $intAnomaly + 1`
      ;;
  esac
  echo "Converting ${blu}${BASH_REMATCH[2]}${BASH_REMATCH[3]}${end} to Unix format..."
  dos2unix $line >/dev/null 2>&1
  done
  echo "C source files processed: ${grn}$intC${end}"
  echo "Cpp source files processed: ${grn}$intCpp${end}"
  echo "Header source files processed: ${grn}$intH${end}"
  echo "Diff files processed: ${grn}$intDiff${end}"
  echo "Lua scripts processed: ${grn}$intLua${end}"
  echo "anomalies encountered: ${red}$intAnomaly${end}"
  duration=$SECONDS
  echo "Processing time: $(($duration / 60)) minutes and $(($duration % 60)) seconds."
  IFS=$OLDIFS
}

#main()
if [ $# -eq 1 ]  #process argument supplied if 1
then
  case "$1" in	# switch case statement
  "--help")
      echo -e $help
      exit 0
      ;;
  "--version")
      echo $0" version "$version
      exit 0
      ;;
  *)
      #default
      sourceroot=$1
      ;;
  esac
fi
if [ $# -gt 1 ]  #too many arguments supplied
then
  echo -e $help
  exit 1 #error level 1 indicates too many arguments supplied
fi
#inform user if running root y/n
if isRoot; then echo "$0 running as root"; else echo "$0 not running as root"; fi
#check if the supplied dir exists
if ! isDirProcessable $sourceroot
then
  echo $sourcerootError 1>&2
  exit 2 #error level 2 indicates sourceroot error
else
  echo "$sourceroot is processable"
fi
#check if the dos2unix dependency is fine
if ! isDependencyOk $program
then
  echo $dependencyError 1>&2
  exit 3 #error level 3 indicates sourceroot error
else
  echo "$program dependency is fine"
fi
processFiles $sourceroot
exit 0
# end script
